import ChannelBase from './channels/channelBase';
import ClientBase from './clients/clientBase';
import PrivateClient from './clients/types/privateClient';
import PrivateChannel from './channels/types/privateChannel';

var logger = require('./logger');

class ChannelManager {
  channels: any = [];

  constructor() {
    // create default channel...
    var channel: ChannelBase = new ChannelBase('default');
    this.channels.push(channel);
  }

  createChannel(data: any) {
    var channelExists: ChannelBase|PrivateChannel|null = this.channelExists(data.channel);
    var channel: ChannelBase|PrivateChannel;

    if (channelExists) {
      channel = channelExists;
      logger.accessLog.info(`Channel already exists: ${channel.id}`);
    } else {
      channel = this.createByChannelType(data);
      this.channels.push(channel);
      logger.accessLog.info(`Added channel to channel manager: ${channel.id}`);
    }

    return channel;
  }

  channelExists(channel_id: string) {
    for (let channel of this.channels) {
      if (channel.id == channel_id) {
        return channel;
      }
    }

    return null;
  }

  addClientToChannel(client: ClientBase|PrivateClient, channel_id: string) {
    var channel: ChannelBase|PrivateChannel|null = this.channelExists(channel_id);

    if (channel) {
      channel.addClient(client);
      client.connectToChannel(channel);
      return {'status': 'success'};
    } else {
      logger.accessLog.info(`channel with id ${channel_id} does not exist.`);
      return {'status': 'notice', 'message': `channel with id ${channel_id} does not exist.`};
    }
  }

  createByChannelType(data: any) {
    var channel_name = data.channel_type + 'Channel'

    try {
      var Channel = require(`./channels/types/${channel_name}`);
      logger.accessLog.info(`attempting to create channel of type ${channel_name}, channel id: ${data.channel}...`);
      return new Channel(data.channel);
    } catch (e) {
      logger.errorLog.info(e);
      logger.accessLog.info(`creating base channel: ${data.channel}`);
      return new ChannelBase(data.channel);
    }
  }

  removeClientFromChannel(client_id: number, channel_id: string) {
    for (let channel of this.channels) {
      if (channel.id == channel_id) {
        if (channel.removeClient(client_id)) {
          logger.accessLog.info(`client removed from channel - channel: ${channel_id}, client: ${client_id}`);
          return true;
        } else {
          logger.errorLog.info(`client not removed from channel, or doesn't exist in channel - channel: ${channel_id}, client: ${client_id}`);
          return false
        }
      }
    }

    return false;
  }

  changeChannel(client: ClientBase|PrivateClient, changeRequest: any) {
    if (client.channel != null) {
      this.removeClientFromChannel(client.id, client.channel.id)
    }

    var channel = this.createChannel(changeRequest);
    this.addClientToChannel(client, channel.id);
    var message = {message_type: 'requestExamStatus'};
    channel.broadcastMessage(client, message);
    this.purgeEmptyChannels();
  }

  purgeEmptyChannels() {
    for (let channel of this.channels) {
      if (channel.clients.length == 0 && channel.id != 'default') {
        var index = this.channels.indexOf(channel)
        this.channels.splice(index, 1);
        logger.accessLog.info(`channel removed: ${channel.id}`);
      }
    }
  }
};

export default ChannelManager;
